/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Application.h"

#include "Config.h"
#include "ConfigurationDialog.h"
#include "CvsCommandDialog.h"
#include "LogDialog.h"
#include "MainWindow.h"
#include "Menu.h"
#include "QuestionDialog.h"
#include "QtUtil.h"
#include "ScratchFileMonitor.h"
#include "Util.h"
#include "XmlFileList.h"
#include "XmlOptions.h"

#include <QApplication>

//____________________________________________
Application::Application( CommandLineArguments arguments ) :
    BaseApplication( 0, arguments ),
    Counter( "Application" )
{}

//____________________________________________
bool Application::initApplicationManager( void )
{
    Debug::Throw( "Application::initApplicationManager.\n" );

    // retrieve files from arguments and expand if needed
    CommandLineParser parser( commandLineParser( _arguments() ) );
    QStringList& orphans( parser.orphans() );
    for( QStringList::iterator iter = orphans.begin(); iter != orphans.end(); ++iter )
    { if( !iter->isEmpty() ) (*iter) = File( *iter ).expand(); }

    // replace arguments
    _setArguments( parser.arguments() );

    // base class initialization
    return BaseApplication::initApplicationManager();

}

//____________________________________________
bool Application::realizeWidget( void )
{
    Debug::Throw( "Application::realizeWidget.\n" );

    // check if the method has already been called.
    if( !BaseApplication::realizeWidget() ) return false;

    // scratch files
    scratchFileMonitor_ = new ScratchFileMonitor( this );
    connect( qApp, SIGNAL(aboutToQuit()), scratchFileMonitor_, SLOT(deleteScratchFiles()) );

    // recent files
    recentFiles_ = new XmlFileList();
    recentFiles_->setCheck( true );

    // insert log list
    mainWindow_ = new MainWindow();
    mainWindow_->centerOnDesktop();
    mainWindow_->setWindowTitle( "Cvswatch" );
    mainWindow_->show();

    QStringList filenames( commandLineParser( _arguments() ).orphans() );
    if( !filenames.isEmpty() ) mainWindow_->open( FileRecord( File( filenames.front() ).expand() ) );
    else mainWindow_->open( FileRecord( Util::workingDirectory() ) );

    emit configurationChanged();

    return true;

}

//____________________________________________
void Application::usage( void ) const
{
    _usage( "cvswatch", tr( "[options] [file]" ) );
    commandLineParser().usage();
    return;
}

//___________________________________________________________
void Application::busy( void )
{
    if( !busyCount_ ) mainWindow_->busy();
    busyCount_ ++;
}

//____________________________________________________________
void Application::idle( void )
{
    if( busyCount_ > 0 ) busyCount_ --;
    if( !busyCount_ ) mainWindow_->idle();
}

//___________________________________________________________
void Application::_configuration( void )
{
    Debug::Throw( "Application::configuration.\n" );
    emit saveConfiguration();
    ConfigurationDialog dialog;
    connect( &dialog, SIGNAL(configurationChanged()), SIGNAL(configurationChanged()) );
    connect( &dialog, SIGNAL(clearCache()), &mainWindow(), SLOT(clearCache()) );
    dialog.centerOnWidget( qApp->activeWindow() );
    dialog.exec();
}

//________________________________________________
bool Application::_processCommand( Server::ServerCommand command )
{

    Debug::Throw( "Application::_processCommand.\n" );
    if( BaseApplication::_processCommand( command ) ) return true;
    if( command.command() == Server::ServerCommand::Raise )
    {
        if( mainWindow_ ) mainWindow_->uniconify();

        QStringList filenames( commandLineParser( command.arguments() ).orphans() );
        if( !filenames.isEmpty() )
        {

            const QString buffer = QString( tr( "Accept request for file '%1' ?" ) ).arg( filenames.front() );
            if( QuestionDialog( mainWindow_, buffer ).centerOnParent().exec() )
            { mainWindow_->open( File( filenames.front() ) ); }
        }

        return true;
    } else return false;

}
