/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ConfigurationDialog.h"

#include "Application.h"
#include "BaseIconNames.h"
#include "BaseFileInfoConfigurationWidget.h"
#include "Debug.h"
#include "GridLayout.h"
#include "IconEngine.h"
#include "OptionSpinBox.h"
#include "OptionBrowsedLineEditor.h"
#include "OptionCheckBox.h"
#include "QuestionDialog.h"
#include "RecentFilesConfiguration.h"
#include "Singleton.h"

#include <QGroupBox>
#include <QLabel>
#include <QPushButton>

//_________________________________________________________
ConfigurationDialog::ConfigurationDialog( QWidget* parent ):
    BaseConfigurationDialog( parent )
{


    Debug::Throw( "ConfigurationDialog::ConfigurationDialog.\n" );
    setWindowTitle( tr( "Configuration - Cvswatch" ) );

    // base configuration
    baseConfiguration();

    // main window
    QWidget* page;
    OptionCheckBox* checkbox;

    page = &addPage( IconEngine::get( IconNames::PreferencesAppearance), tr( "Appearance" ), tr( "Main window appearance" ) );
    QWidget *box = new QWidget( page );
    page->layout()->addWidget( box );

    QVBoxLayout* layout = new QVBoxLayout();
    layout->setMargin(0);
    layout->setSpacing(5);
    box->setLayout( layout );

    GridLayout* gridLayout = new GridLayout();
    gridLayout->setSpacing(5);
    gridLayout->setMargin(5);
    gridLayout->setMaxCount(2);
    layout->addLayout( gridLayout );

    // icon view pixmap size
    QLabel* label;
    gridLayout->addWidget( label = new QLabel( tr( "Icon size in icon view:" ), box ) );
    OptionSpinBox *spinbox = new OptionSpinBox( box, "ICON_VIEW_ICON_SIZE" );
    spinbox->setMinimum( 8 );
    spinbox->setMaximum( 128 );
    spinbox->setSuffix( tr( "px" ) );
    gridLayout->addWidget( spinbox );
    addOptionWidget( spinbox );
    label->setBuddy( spinbox );

    // full path
    layout->addWidget( checkbox = new OptionCheckBox( tr( "Show full path inside location bar" ), box, "USE_TRUNCATION" ) );
    checkbox->setNegative( true );
    addOptionWidget( checkbox );

    // navigation
    box->layout()->addWidget( checkbox = new OptionCheckBox( tr( "Show parent directory icon in file list" ), box, "SHOW_NAVIGATOR" ) );
    addOptionWidget( checkbox );

    // hidden files
    layout->addWidget( checkbox = new OptionCheckBox( tr( "Show hidden files" ), box, "SHOW_HIDDEN_FILES" ) );
    addOptionWidget( checkbox );

    // tooltips
    page = &addPage( IconEngine::get( IconNames::PreferencesAppearance), tr( "Tooltips" ), tr( "Tooltips appearance" ) );
    page->layout()->addWidget( checkbox = new OptionCheckBox( tr( "Show tooltips" ), page, "SHOW_TOOLTIPS" ) );
    addOptionWidget( checkbox );

    QHBoxLayout* hLayout = new QHBoxLayout();
    hLayout->setMargin(0);
    hLayout->setSpacing(5);
    page->layout()->addItem( hLayout );

    BaseFileInfoConfigurationWidget* configurationWidget;
    hLayout->addSpacing( _checkBoxSpacing() );
    hLayout->addWidget( configurationWidget = new BaseFileInfoConfigurationWidget( page ) );
    configurationWidget->setEnabled( false );
    connect( checkbox, SIGNAL(toggled(bool)), configurationWidget, SLOT(setEnabled(bool)) );
    addOptionWidget( configurationWidget );

    // recent files
    page = &addPage( IconEngine::get( IconNames::PreferencesRecentFiles ), tr( "Recent files" ), tr( "Recent files list settings" ), true );
    RecentFilesConfiguration* recentFilesConfiguration = new RecentFilesConfiguration( page, Singleton::get().application<Application>()->recentFiles() );
    page->layout()->addWidget( recentFilesConfiguration );
    addOptionWidget( recentFilesConfiguration );

    recentFilesConfiguration->read();
    connect( this, SIGNAL(ok()), recentFilesConfiguration, SLOT(write()) );
    connect( this, SIGNAL(apply()), recentFilesConfiguration, SLOT(write()) );
    connect( this, SIGNAL(reset()), recentFilesConfiguration, SLOT(reload()) );

    // Cache
    page = &addPage( IconEngine::get( IconNames::PreferencesCache ), tr( "Cache" ), tr( "Configure the cache used to store CVS information" ) );

    //
    box = new QWidget( page );
    page->layout()->addWidget( box );

    hLayout = new QHBoxLayout();
    hLayout->setSpacing(5);
    hLayout->setMargin(5);
    box->setLayout( hLayout );

    layout = new QVBoxLayout();
    layout->setMargin(0);
    layout->setSpacing(5);
    hLayout->addLayout( layout );
    hLayout->addStretch();

    gridLayout = new GridLayout();
    gridLayout->setSpacing(5);
    gridLayout->setMargin(0);
    gridLayout->setMaxCount(2);
    layout->addLayout( gridLayout );

    gridLayout->addWidget( checkbox = new OptionCheckBox( tr( "Limit revision cache size to") , box, "LIMIT_REVISION_CACHE" ) );
    checkbox->setToolTip( tr( "Enable/disable the limitation of the file revision cache to a maximum size" ) );
    addOptionWidget( checkbox );

    gridLayout->addWidget( spinbox = new OptionSpinBox( box, "REVISION_CACHE_SIZE" ) );
    spinbox->setMinimum( 0 );
    spinbox->setMaximum( 1000 );
    spinbox->setSuffix( tr( "MiB" ) );
    spinbox->setToolTip( tr( "Maximum size of the revision cache" ) );
    addOptionWidget( spinbox );

    gridLayout->addWidget( checkbox = new OptionCheckBox( tr( "Limit file information cache size to" ), box, "LIMIT_FILEINFO_CACHE" ) );
    checkbox->setToolTip( tr( "Enable/disable the limitation of the file information cache to a maximum size" ) );
    addOptionWidget( checkbox );

    gridLayout->addWidget( spinbox = new OptionSpinBox( box, "FILEINFO_CACHE_SIZE" ) );
    spinbox->setMinimum( 0 );
    spinbox->setMaximum( 1000 );
    spinbox->setSuffix( tr( "MiB" ) );
    spinbox->setToolTip( tr( "Maximum size of the revision cache" ) );
    addOptionWidget( spinbox );

    layout->addWidget( label = new QLabel( tr( "<a href='clear'><u>Clear recent file information and revisions cache...</u></a>" ), box ) );
    connect( label, SIGNAL(linkActivated(QString)), SLOT(_clearCache()) );

    // commands
    page = &addPage( IconEngine::get( IconNames::PreferencesFileAssociations ), tr( "Applications" ), tr( "Third-party applications" ) );
    box = new QGroupBox( page );
    page->layout()->addWidget( box );

    gridLayout = new GridLayout();
    gridLayout->setSpacing(5);
    gridLayout->setMargin(5);
    gridLayout->setMaxCount(2);
    gridLayout->setColumnAlignment( 0, Qt::AlignRight | Qt::AlignVCenter );
    box->setLayout( gridLayout );

    OptionBrowsedLineEditor* editor;
    gridLayout->addWidget( label = new QLabel( tr( "Cvs:" ), box ) );
    gridLayout->addWidget( editor = new OptionBrowsedLineEditor( box, "CVS" ) );
    addOptionWidget( editor );
    label->setBuddy( editor );

    gridLayout->addWidget( label = new QLabel( tr( "Text editor:" ), box ) );
    gridLayout->addWidget( editor = new OptionBrowsedLineEditor( box, "EDITOR" ) );
    addOptionWidget( editor );
    label->setBuddy( editor );

    gridLayout->addWidget( label = new QLabel( tr( "Diff text editor:" ), box ) );
    gridLayout->addWidget( editor = new OptionBrowsedLineEditor( box, "DIFF_EDITOR" ) );
    addOptionWidget( editor );
    label->setBuddy( editor );

    box = new QGroupBox( tr( "Environment" ), page );
    page->layout()->addWidget( box );

    gridLayout = new GridLayout();
    gridLayout->setSpacing(5);
    gridLayout->setMargin(5);
    gridLayout->setMaxCount(2);
    gridLayout->setColumnAlignment( 0, Qt::AlignRight | Qt::AlignVCenter );
    box->setLayout( gridLayout );

    gridLayout->addWidget( label = new QLabel( tr( "CVS root:" ), box ) );
    gridLayout->addWidget( editor = new OptionBrowsedLineEditor( box, "DEFAULT_CVSROOT" ) );
    addOptionWidget( editor );
    label->setBuddy( editor );

    // load initial configuration
    read();

}

//_________________________________________________________
void ConfigurationDialog::_clearCache( void )
{
    if( !QuestionDialog( this, tr( "Permanently clear revisions and file informations caches ?" ) ).exec() ) return;
    emit clearCache();
}
