/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "CvsCommandModel.h"

#include "IconNames.h"
#include "IconEngine.h"
#include "Singleton.h"
#include "TimeStamp.h"
#include "XmlOptions.h"

#include <QIcon>

//_______________________________________________
const QString CvsCommandModel::columnTitles_[ CvsCommandModel::nColumns ] =
{
    tr( "Id" ),
    tr( "Command" ),
    tr( "Path" )
};

//_______________________________________________
CvsCommandModel::IconCache& CvsCommandModel::_icons( void )
{
    static IconCache singleton;
    return singleton;
}

//_______________________________________________________________
CvsCommandModel::CvsCommandModel( QObject* parent ):
    ListModel<CvsCommand>( parent ),
    Counter( "CvsCommandModel" )
{

    Debug::Throw( "CvsCommandModel::CvsCommandModel.\n" );
    connect( Singleton::get().application(), SIGNAL(configurationChanged()), SLOT(_updateConfiguration()) );
    _updateConfiguration();

}

//__________________________________________________________________
QVariant CvsCommandModel::data( const QModelIndex& index, int role ) const
{
    Debug::Throw( "CvsCommandModel::data.\n" );

    // check index, role and column
    if( !index.isValid() ) return QVariant();

    // retrieve associated command
    const CvsCommand& command( get()[index.row()] );

    // return text associated to command and column
    if( role == Qt::DisplayRole ) {

        switch( index.column() )
        {
            case Id:
            return command.id();

            case Name:
            return QString( command.name() );

            case Path:
            return command.argumentCount() > 0  ? command.argument(0):QVariant();

            default:
            return QVariant();
        }
    }

    // return icon associated to file
    if( role == Qt::DecorationRole && index.column() == Name )
    { return _icons()[command.type()]; }

    return QVariant();

}

//__________________________________________________________________
QVariant CvsCommandModel::headerData(int section, Qt::Orientation orientation, int role) const
{

    if(
        orientation == Qt::Horizontal &&
        role == Qt::DisplayRole &&
        section >= 0 &&
        section < nColumns )
    { return columnTitles_[section]; }

    // return empty
    return QVariant();

}

//____________________________________________________________
void CvsCommandModel::add( const CvsCommand& command )
{
    Debug::Throw( "CvsCommandModel::add.\n" );
    emit layoutAboutToBeChanged();
    _get() << command;
    emit layoutChanged();
}

//____________________________________________________________
void CvsCommandModel::remove( const CvsCommand& command )
{

    Debug::Throw( "CvsCommandModel::remove.\n" );

    List::iterator iter = std::find_if( _get().begin(), _get().end(), CvsCommand::SameTypeFTor( command.type() ) );
    if( iter != _get().end() )
    {
        emit layoutAboutToBeChanged();
        _get().erase( iter );
        emit layoutChanged();
    }

    return;

}

//_________________________________________________________
void CvsCommandModel::_updateConfiguration( void )
{

    Debug::Throw( "CvsCommandModel::_updateConfiguration.\n" );

    beginResetModel();

    // reinitialize icons
    _icons().clear();
    _installIcons();

    endResetModel();

}

//_____________________________________________________________________
void CvsCommandModel::_installIcons( void ) const
{

    Debug::Throw( "CvsCommandList::_installIcons" );

    if( !_icons().empty() ) return;

    // load pixmaps
    const unsigned int pixmapSize = XmlOptions::get().get<unsigned int>( "LIST_ICON_SIZE" );

    QSize size( pixmapSize, pixmapSize );

    // type icons
    using IconNamesHash=QHash< CvsCommand::Type, QString >;
    IconNamesHash typeNames;
    typeNames[CvsCommand::Status] = IconNames::RemotelyModified;
    typeNames[CvsCommand::Log] = IconNames::RemotelyModified;
    typeNames[CvsCommand::FakeUpdate] = IconNames::RemotelyModified;
    typeNames[CvsCommand::Checkout] = IconNames::RemotelyAdded;
    typeNames[CvsCommand::Update] = IconNames::RemotelyModified;
    typeNames[CvsCommand::Add] = IconNames::LocallyAdded;
    typeNames[CvsCommand::Remove] = IconNames::LocallyRemoved;
    typeNames[CvsCommand::Commit] = IconNames::LocallyModified;
    typeNames[CvsCommand::CheckoutRevision] = IconNames::RemotelyAdded;

    // load link pixmap
    for( IconNamesHash::iterator iter = typeNames.begin(); iter != typeNames.end(); ++iter )
    { _icons().insert( iter.key(), IconEngine::get( iter.value() ) ); }

    return;
}
