/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "FileIconProvider.h"
#include "FileInfo.h"

#include "CustomPixmap.h"
#include "IconNames.h"
#include "IconEngine.h"

//__________________________________________________________________
FileIconProvider::FileIconProvider( QObject* parent ):
    BaseFileIconProvider( parent )
{

    // status icons, for all documents
    statusNames_[FileInfo::Unknown] = IconNames::Unknown;
    statusNames_[FileInfo::UpToDate] = IconNames::UpToDate;
    statusNames_[FileInfo::LocallyModified] = IconNames::LocallyModified;
    statusNames_[FileInfo::LocallyAdded] = IconNames::LocallyAdded;
    statusNames_[FileInfo::LocallyRemoved] = IconNames::LocallyRemoved;
    statusNames_[FileInfo::RemotelyModified] = IconNames::RemotelyModified;
    statusNames_[FileInfo::RemotelyAdded] = IconNames::RemotelyAdded;
    statusNames_[FileInfo::RemotelyRemoved] = IconNames::RemotelyRemoved;
    statusNames_[FileInfo::Merge] = IconNames::Merge;
    statusNames_[FileInfo::Conflict] = IconNames::Conflict;

}

//__________________________________________________________________
const QIcon& FileIconProvider::icon( const BaseFileInfo& fileInfo )
{

    // try from base class
    const QIcon& icon( BaseFileIconProvider::icon( fileInfo ) );
    if( !icon.isNull() ) return icon;

    // get relevant file info type
    int type( fileInfo.type() );
    int status( type >> FileInfo::Shift );

    type &= (FileInfo::Document|FileInfo::Folder|FileInfo::Link|FileInfo::Hidden|FileInfo::Navigator);
    type |= status << FileInfo::Shift;

    // build key
    Key key( fileInfo.file().extension(), type );

    // check if already created
    IconCache::const_iterator iter( icons_.find( key ) );
    if( iter != icons_.end() ) return iter.value();

    // create
    QIcon out;
    if( type & FileInfo::Navigator ) out = IconEngine::get( IconNames::Parent );
    else if( type & FileInfo::Folder ) out = IconEngine::get( IconNames::Folder );
    else if( type & FileInfo::Document )
    {

        // try get icon from mimetypes
        out = mimeTypeIconProvider_.icon( fileInfo.file().extension() );
        if( out.isNull() ) out = IconEngine::get( IconNames::Document );

    }

    // add status
    if( (type & FileInfo::Document) && statusNames_.contains( status ) )
    {

        QIcon copy( out );
        out = QIcon();
        const CustomPixmap statusOverlay( CustomPixmap().find( statusNames_[status] ) );
        foreach( const QSize& size, copy.availableSizes() )
        {
            out.addPixmap(
                CustomPixmap( copy.pixmap( size ) ).merge(
                statusOverlay.scaled( 0.4*size, Qt::KeepAspectRatio, Qt::SmoothTransformation ) ) );
        }

    }

    // insert in map and return
    return icons_.insert( key, out ).value();

}
