#ifndef FileInfo_h
#define FileInfo_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "BaseFileInfo.h"
#include "QOrderedSet.h"

#include <QDomDocument>
#include <QDomElement>

//! store a file and needed informations for Ftp transfer
class FileInfo: public BaseFileInfo, public Counter
{

    public:

    //! cvs status
    enum Status
    {
        Unknown = 0,
        UpToDate = 1<<0,
        LocallyModified = 1<<1,
        LocallyAdded = 1<<2,
        LocallyRemoved = 1<<3,
        RemotelyModified = 1<<4,
        RemotelyAdded = 1<<5,
        RemotelyRemoved = 1<<6,
        Merge = 1<<7,
        Conflict = 1<<8,
    };

    Q_DECLARE_FLAGS( StatusFlags, Status )

    //! constructor
    FileInfo( File = File(), TypeFlags = None );

    //! constructor
    FileInfo( File, TypeFlags, StatusFlags );

    //! constructor from DOM element
    FileInfo( const QDomElement& );

    //! dump to dom element
    QDomElement domElement( QDomDocument& parent ) const;


    //! equal to operator
    virtual bool operator == (const BaseFileInfo& other) const
    { return file() == other.file() && BaseFileInfo::type() == other.BaseFileInfo::type(); }

    //!@name accessors
    //@{

    //! creation
    time_t created( void ) const
    { return creation_; }

    //! true when file and local information are not synchronized
    bool dirty( void ) const
    { return dirty_; }

    //! file type
    virtual int type( void ) const
    { return BaseFileInfo::type() | (status() << Shift ); }

    //! get status
    StatusFlags status( void ) const
    { return status_; }

    //! status string
    static QString statusString( StatusFlags );

    //! status string
    QString statusString( void ) const
    { return statusString( status_ ); }

    //@}

    //!@name modifiers
    //@{

    //! file
    virtual void setFile( const File& file )
    {
        BaseFileInfo::setFile( file );
        setDirty( true );
    }

    //! dirty
    void setDirty( bool value )
    { dirty_ = value; }

    //! type
    virtual void setType( TypeFlags type )
    { BaseFileInfo::setType( type ); }

    //! set status
    void setStatus( StatusFlags status )
    { status_ = status; }

    //! time stamp
    void setCreation( time_t value )
    { creation_ = value; }

    //! update internal storage from file
    virtual void update( void );

    //! update internal storage from existing file info
    virtual void updateFrom( const FileInfo& );

    //@}

    //! needed to sort fileInfo by creation timeStamp
    class FirstCreatedFTor
    {
        public:

        //! sort operator
        bool operator() (const FileInfo& first, const FileInfo& second ) const
        { return first.created() < second.created(); }


    };

    //! used to count files that do not match a given type
    class DifferentTypeFTor
    {
        public:

        //! constructor
        DifferentTypeFTor( int type ):
            type_( type )
        {}

        //! predicate
        /*! the construct (info.type()&type_) == type_ enables to use bit pattern for a type */
        bool operator() (const FileInfo& info ) const
        { return !(info.type()&type_); }

        private:

        //! predicted type
        int type_;

    };

    //! needed to retrieve items that are documents
    class ExistsFTor
    {

        public:

        //! constructor
        ExistsFTor( const File& path ):
            path_( path )
        {}

        //! prediction
        bool operator() ( const FileInfo& info ) const
        { return !( info.type() & FileInfo::Navigator ) && info.file().addPath( path_ ).exists(); }

        private:

        //! base path
        File path_;

    };

    //! needed to retrieve items that are documents
    class IsExistingDocumentFTor
    {

        public:

        //! constructor
        IsExistingDocumentFTor( const File& path ):
            path_( path )
        {}

        //! prediction
        bool operator() ( const FileInfo& info ) const
        { return  ( info.type() & FileInfo::Document ) && info.file().addPath( path_ ).exists(); }

        private:

        //! base path
        File path_;

    };

    //@}

    //! list
    using List = BaseFileInfo::BaseList<FileInfo>;

    //! set
    using Set = QOrderedSet<FileInfo>;

    private:

    //! true if file and information are not synchronized
    bool dirty_;

    //! file type
    StatusFlags status_;

    //! creation time
    time_t creation_;

};

Q_DECLARE_OPERATORS_FOR_FLAGS( FileInfo::StatusFlags)

#endif
