#ifndef FileSelectionFrame_h
#define FileSelectionFrame_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "CvsCommand.h"
#include "File.h"
#include "FileInfo.h"
#include "FileInfoCache.h"
#include "FileInfoModel.h"
#include "FileSystemWatcher.h"
#include "FileThread.h"
#include "TextFormat.h"

#include <QBasicTimer>
#include <QAbstractItemView>
#include <QItemSelectionModel>
#include <QIcon>
#include <QMenu>
#include <QWidget>

// forward declarations
class PathEditor;
class BaseFileInfoToolTipWidget;
class CvsInterface;

//* customized ListView for file/directory navigation
class FileSelectionFrame: public QWidget, public Counter
{

    //* Qt meta object declaration
    Q_OBJECT

    public:

    //* constructor
    FileSelectionFrame( QWidget* parent );

    //* editor
    PathEditor& editor( void ) const
    { return *editor_; }

    //* base directory
    const File& path( void ) const
    { return path_; }

    //* clear list, add navigator button
    void clear();

    //* cvs interface
    CvsInterface& interface( void ) const
    { return *interface_; }

    //* list
    QAbstractItemView& list( void ) const
    { return *list_; }

    //*@name mode
    //@{

    enum ViewMode
    {
        List,
        Icons
    };

    //* mode
    ViewMode viewMode( void ) const
    { return viewMode_; }

    //@}

    //* selected files
    FileInfo::List selection( void ) const;

    //* returns true if there are selected documents
    bool hasSelectedDocuments( bool alsoExist ) const;

    //* cache
    FileInfoCache& cache( void )
    {
        cache_.merge( localCache_ );
        return cache_;
    }

    //* cache
    void setCache( const FileInfoCache& cache )
    { cache_ = cache; }

    //* model
    const FileInfoModel& model( void ) const
    { return model_; }

    //* model
    FileInfoModel& model( void )
    { return model_; }

    //*@name actions
    //@{

    //* hidden files
    QAction& hiddenFilesAction( void ) const
    { return *hiddenFilesAction_; }

    //* edit file
    QAction& editAction( void ) const
    { return *editAction_; }

    //* remove local file
    QAction& removeAction( void ) const
    { return *removeAction_; }

    //* previous directory
    QAction& previousDirectoryAction( void ) const
    { return *previousDirectoryAction_; }

    //* next directory
    QAction& nextDirectoryAction( void ) const
    { return *nextDirectoryAction_; }

    // directory above
    QAction& parentDirectoryAction( void ) const
    { return *parentDirectoryAction_; }

    // home directory
    QAction& homeDirectoryAction( void ) const
    { return *homeDirectoryAction_; }

    //* cvs checkout
    QAction& cvsCheckOutAction( void ) const
    { return *cvsCheckoutAction_; }

    //* cvs update
    QAction& cvsUpdateAction( void ) const
    { return *cvsUpdateAction_; }

    //* cvs add
    QAction& cvsAddAction( void ) const
    { return *cvsAddAction_; }

    //* cvs remove
    QAction& cvsRemoveAction( void ) const
    { return *cvsRemoveAction_; }

    //* cvs commit
    QAction& cvsCommitAction( void ) const
    { return *cvsCommitAction_; }

    //* cvs update directory
    QAction& cvsUpdateDirectoryAction( void ) const
    { return *cvsUpdateDirectoryAction_; }

    //* cvs commit directory
    QAction& cvsCommitDirectoryAction( void ) const
    { return *cvsCommitDirectoryAction_; }

    //* file properties
    QAction& filePropertiesAction( void ) const
    { return *filePropertiesAction_; }

    //@}

    Q_SIGNALS:

    //* emitted for logging of processes
    void messageAvailable( const QString&, Format::TextFormatFlags = 0, const QColor& = QColor(), int = 0 );

    //* selection changed
    void selectionChanged( void );

    //* file selected. Update RevisionFrame. Do not checkout revisions
    void fileSelected( File );

    //* request file information
    void requestFileInformation( const FileInfo&, const QIcon& );

    public Q_SLOTS:

    //* mode
    void setViewMode( FileSelectionFrame::ViewMode );

    //* base directory
    void setFile( const File& );

    //* reload (without changing the path)
    void reload( void )
    { _reload( false ); }

    //* a command just completed
    void commandCompleted( const CvsCommand& );

    //* clear caches
    void clearCache( void )
    {
        cache_.clear();
        localCache_.clear();
    }

    protected:

    //* timer event
    virtual void timerEvent( QTimerEvent* );

    //* setup list
    void _setupList( void );

    protected Q_SLOTS:

    //* update configuration
    void _updateConfiguration( void );

    //* show hidden files
    void _toggleShowHiddenFiles( bool );

    //* navigator
    void _setShowNavigator( bool );

    //* an item in the list have been selected
    /** if its a file, update the revision frame */
    void _itemSelected( const QModelIndex& );

    //* an item in list have been activated
    /** either change directory or request revisions */
    void _itemActivated( const QModelIndex& );

    //* update list menu
    void _updateContextMenu( const QPoint& );

    //* update action state depending on selection
    void _updateActions( void );

    //* update navigation actions based on current location and history
    void _updateNavigationActions( void );

    //* show tooltip
    void _showToolTip( const QModelIndex& );

    //* reload directory
    void _reload( bool clearFirst = true, bool checkCvs = true );

    //* select local file if any
    void _selectFile( void );

    //* edit selected files
    void _edit( void );

    //* remove (locally) selected files
    void _remove( void );

    //* previous directory
    void _previousDirectory( void );

    //* next directory
    void _nextDirectory( void );

    //* parent directory
    void _parentDirectory( void );

    //* home directory
    void _homeDirectory( void );

    //* show status
    void _showStatus( File, FileInfo::List );

    //* cvs checkout
    void _cvsCheckOut( void );

    //* cvs update
    void _cvsUpdate( void );

    //* cvs update directory
    void _cvsUpdateDirectory( void );

    //* cvs add
    void _cvsAdd( void );

    //* cvs add
    void _cvsRemove( void );

    //* cvs commit
    void _cvsCommit( void );

    //* cvs commit
    void _cvsCommitDirectory( void );

    //* toggle show navigator
    void _toggleShowNavigator( bool );

    //* process files
    void _processFiles( const File::List& );

    //* file properties
    void _fileProperties( void );

    private:

    //* install actions
    void _installActions( void );

    //* show status from cache
    void _showStatusFromLocalCache( void );

    //* edit file
    void _edit( File );

    //* model
    FileInfoModel model_;

    //* selection model
    QItemSelectionModel selectionModel_;

    //* file editor
    PathEditor* editor_ = nullptr;

    //* tooltip widget
    BaseFileInfoToolTipWidget* toolTipWidget_ = nullptr;

    //* list of files
    QAbstractItemView* list_ = nullptr;

    //* view mode option name
    QString viewModeOptionName_;

    //* mode
    ViewMode viewMode_ = Icons;

    //* current path
    File path_;

    //* current file
    File file_;

    //* timer
    /**
    used to update status from CVS interface,
    with some delay to avoid running too many
    CVS commands when parsing the tree
    */
    QBasicTimer timer_;

    //*@name menus
    //@{

    //* column selection
    QMenu* columnSelectionMenu_ = nullptr;

    //* column sorting
    QMenu* columnSortingMenu_ = nullptr;

    //@}

    //*@name actions
    //@{

    //* show/hide hidden files
    QAction* hiddenFilesAction_ = nullptr;

    //* edit
    QAction* editAction_ = nullptr;

    //* remove
    QAction* removeAction_ = nullptr;

    //* previous directory action
    QAction* previousDirectoryAction_ = nullptr;

    //* next directory action
    QAction* nextDirectoryAction_ = nullptr;

    //* parent directory action
    QAction* parentDirectoryAction_ = nullptr;

    //* home directory action
    QAction* homeDirectoryAction_ = nullptr;

    //* cvs checkout
    QAction* cvsCheckoutAction_ = nullptr;

    //* cvs update
    QAction* cvsUpdateAction_ = nullptr;

    //* cvs update directory
    QAction* cvsUpdateDirectoryAction_ = nullptr;

    //* cvs add
    QAction* cvsAddAction_ = nullptr;

    //* cvs remove
    QAction* cvsRemoveAction_ = nullptr;

    //* cvs commit
    QAction* cvsCommitAction_ = nullptr;

    //* cvs commit
    QAction* cvsCommitDirectoryAction_ = nullptr;

    //* file properties
    QAction* filePropertiesAction_ = nullptr;

    //@}

    //* cvs interface
    CvsInterface* interface_ = nullptr;

    //* file thread
    FileThread thread_;

    //* File-info full cache
    FileInfoCache cache_;

    //* File-info local cache (matching current path)
    FileInfoCache localCache_;

    //* file system watcher, for local selection frame
    FileSystemWatcher fileSystemWatcher_;

    //* check cvs
    bool checkCvs_ = true;

    //* show navigator
    bool showNavigator_ = true;

};

#endif
