/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "MainWindow.h"
#include "Application.h"
#include "BaseStatusBar.h"
#include "ClockLabel.h"
#include "Command.h"
#include "FileDialog.h"
#include "CustomToolBar.h"
#include "CvsCommandDialog.h"
#include "DockWidget.h"
#include "FileInfoCacheDialog.h"
#include "FileIconProvider.h"
#include "FileInformationDialog.h"
#include "FileList.h"
#include "FileSelectionFrame.h"
#include "IconNames.h"
#include "IconEngine.h"
#include "InformationDialog.h"
#include "LogDialog.h"
#include "LogWidget.h"
#include "Menu.h"
#include "PathEditor.h"
#include "PlacesWidget.h"
#include "QtUtil.h"
#include "RecentFilesMenu.h"
#include "RevisionCacheDialog.h"
#include "RevisionFrame.h"
#include "ScratchFileMonitor.h"
#include "Singleton.h"
#include "TreeView.h"
#include "Util.h"
#include "ViewModeMenu.h"
#include "XmlOptions.h"

#include <QApplication>
#include <QLabel>
#include <QGroupBox>
#include <QSplitter>

//___________________________________________________________
MainWindow::MainWindow( QWidget* parent ):
    BaseMainWindow( parent ),
    Counter( "MainWindow" )
{

    setOptionName( "MAIN_WINDOW" );

    Debug::Throw( "MainWindow::MainWindow.\n" );
    _installActions();
    Application& application( *Singleton::get().application<Application>() );
    addAction( &application.closeAction() );

    // main widget and layout
    QWidget* main = new QWidget( this );
    QVBoxLayout* mainLayout( new QVBoxLayout() );
    main->setLayout( mainLayout );
    mainLayout->setMargin(0);
    mainLayout->setSpacing(0);
    setCentralWidget( main );

    // splitter
    QSplitter *splitter = new QSplitter( main );
    mainLayout->addWidget( splitter, 1 );
    splitter->setOrientation( Qt::Horizontal );

    //* dialogs
    commandDialog_ = new CvsCommandDialog();
    logDialog_ = new LogDialog();

    connect( commandDialog_, SIGNAL(dialogHidden()), SLOT(_commandDialogHidden()) );
    connect( logDialog_, SIGNAL(dialogHidden()), SLOT(_logDialogHidden()) );

    // file selection frame on the left
    fileSelectionFrame_ = new FileSelectionFrame( splitter );
    fileSelectionFrame_->editor().setParent( main );
    mainLayout->insertWidget( 0, &fileSelectionFrame_->editor() );

    // create icon provider and assign
    iconProvider_ = new FileIconProvider( this );
    fileSelectionFrame_->model().setIconProvider( iconProvider_ );

    // second splitter on the right
    revisionFrame_ = new RevisionFrame( splitter );

    // menu bar
    setMenuBar( menu_ = new Menu( this ) );
    connect( &menu_->viewModeMenu(), SIGNAL(viewModeSelected(FileSelectionFrame::ViewMode)), fileSelectionFrame_, SLOT(setViewMode(FileSelectionFrame::ViewMode)) );

    // status bar
    setStatusBar( statusbar_ = new BaseStatusBar( this ) );
    statusbar_->addLabels(1, 1 );
    statusbar_->addClock();

    // navigation toolbar
    CustomToolBar* toolbar;
    toolbar = new CustomToolBar( "Navigation", this, "NAVIGATION_TOOLBAR" );
    toolbar->addAction( &fileSelectionFrame_->homeDirectoryAction() );
    toolbar->addAction( &fileSelectionFrame_->previousDirectoryAction() );
    toolbar->addAction( &fileSelectionFrame_->nextDirectoryAction() );
    toolbar->addAction( &fileSelectionFrame_->parentDirectoryAction() );
    toolbar->addAction( &reloadAction() );

    toolbar->addSeparator();
    toolbar->addAction( &fileSelectionFrame_->cvsUpdateDirectoryAction() );
    toolbar->addAction( &fileSelectionFrame_->cvsCheckOutAction() );

    toolbar->addSeparator();
    toolbar->addAction( &fileSelectionFrame_->cvsAddAction() );
    toolbar->addAction( &fileSelectionFrame_->cvsRemoveAction() );
    toolbar->addAction( &fileSelectionFrame_->cvsCommitDirectoryAction() );

    toolbar->addSeparator();
    toolbar->addAction( &fileSelectionFrame_->filePropertiesAction() );

    // dock widgets
    placesPanel_ = new DockWidget( tr( "Places" ), this, "PLACES" );
    placesPanel_->setAllowedAreas( Qt::LeftDockWidgetArea|Qt::RightDockWidgetArea );
    placesPanel_->setUseScrollArea( true );
    placesPanel_->visibilityAction().setText( tr( "Show Favorite Places" ) );
    addDockWidget( Qt::LeftDockWidgetArea, placesPanel_ );

    placesWidget_ = new PlacesWidget();
    placesWidget_->setIconProvider( iconProvider_ );
    placesPanel_->setMainWidget( placesWidget_ );

    if( !placesPanel_->visibilityAction().isChecked() )
    { placesPanel_->hide(); }

    // connections
    // file editor
    connect( &fileSelectionFrame_->editor(), SIGNAL(pathChanged(File)), SLOT(_reload()) );

    connect( fileSelectionFrame_, SIGNAL(fileSelected(File)), SLOT(_setFile(File)) );
    connect( fileSelectionFrame_, SIGNAL(requestFileInformation(FileInfo,QIcon)), SLOT(_fileInformation(FileInfo,QIcon)) );
    connect( fileSelectionFrame_, SIGNAL(messageAvailable(QString,Format::TextFormatFlags,QColor,int)), &logDialog().editor(), SLOT(append(QString,Format::TextFormatFlags,QColor,int)) );

    connect( &fileSelectionFrame_->interface(), SIGNAL(commandStarted(CvsCommand)), Singleton::get().application(), SLOT(busy()) );
    connect( &fileSelectionFrame_->interface(), SIGNAL(commandCompleted(CvsCommand)), Singleton::get().application(), SLOT(idle()) );
    connect( &fileSelectionFrame_->interface(), SIGNAL(commandAdded(CvsCommand)), &commandDialog().model(), SLOT(add(CvsCommand)) );
    connect( &fileSelectionFrame_->interface(), SIGNAL(commandRemoved(CvsCommand)), &commandDialog().model(), SLOT(remove(CvsCommand)) );
    connect( &fileSelectionFrame_->interface(), SIGNAL(commandCompleted(CvsCommand)), &commandDialog().model(), SLOT(remove(CvsCommand)) );
    connect( &fileSelectionFrame_->interface(), SIGNAL(commandCompleted(CvsCommand)), revisionFrame_, SLOT(commandCompleted(CvsCommand)) );
    connect( &fileSelectionFrame_->interface(), SIGNAL(messageAvailable(QString,Format::TextFormatFlags,QColor,int)), &logDialog().editor(), SLOT(append(QString,Format::TextFormatFlags,QColor,int)) );
    connect( &fileSelectionFrame_->interface(), SIGNAL(commandStarted(CvsCommand)), SLOT(_updateStatusBar(CvsCommand)) );
    connect( &fileSelectionFrame_->interface(), SIGNAL(commandCompleted(CvsCommand)), &statusbar_->label(), SLOT(clear()) );

    connect( &revisionFrame_->interface(), SIGNAL(messageAvailable(QString,Format::TextFormatFlags,QColor,int)), &logDialog().editor(), SLOT(append(QString,Format::TextFormatFlags,QColor,int)) );
    connect( &revisionFrame_->interface(), SIGNAL(commandStarted(CvsCommand)), Singleton::get().application(), SLOT(busy()) );
    connect( &revisionFrame_->interface(), SIGNAL(commandCompleted(CvsCommand)), Singleton::get().application(), SLOT(idle()) );
    connect( &revisionFrame_->interface(), SIGNAL(commandAdded(CvsCommand)), &commandDialog().model(), SLOT(add(CvsCommand)) );
    connect( &revisionFrame_->interface(), SIGNAL(commandRemoved(CvsCommand)), &commandDialog().model(), SLOT(remove(CvsCommand)) );
    connect( &revisionFrame_->interface(), SIGNAL(commandCompleted(CvsCommand)), &commandDialog().model(), SLOT(remove(CvsCommand)) );

    // places
    connect( placesWidget_, SIGNAL(itemSelected(BaseFileInfo)), SLOT(_placeSelected(BaseFileInfo)) );

    // propagate command completed from revision frame to FileSelectionFrame
    // this to allow diff actions and checkout revision actions to toggle update of the list
    connect( &revisionFrame_->interface(), SIGNAL(commandCompleted(CvsCommand)), fileSelectionFrame_, SLOT(commandCompleted(CvsCommand)) );
    connect( &revisionFrame_->interface(), SIGNAL(commandStarted(CvsCommand)), SLOT(_updateStatusBar(CvsCommand)) );
    connect( &revisionFrame_->interface(), SIGNAL(commandCompleted(CvsCommand)), &statusbar_->label(), SLOT(clear()) );
    connect( &revisionFrame_->interface(), SIGNAL(scratchFileAdded(File)), &Singleton::get().application<Application>()->scratchFileMonitor(), SLOT(add(File)) );

    connect( &application, SIGNAL(configurationChanged()), SLOT(_updateConfiguration()) );
    connect( qApp, SIGNAL(aboutToQuit()), SLOT(_saveConfiguration()) );
    _updateConfiguration();

    // read cache and assign to children
    cache_.read( QString( XmlOptions::get().raw( "CACHE_FILE" ) ) );
    fileSelectionFrame_->setCache( cache_.fileInfoCache() );
    revisionFrame_->setCache( cache_.revisionCache() );

    // splitter
    splitter->setStretchFactor( 0, 1 );
    splitter->setStretchFactor( 1, 1 );

    // busy widget
    busyWidget_ = new BusyWidget( main, BusyWidget::BottomRight );
    busyWidget_->hide();

}

//__________________________________________________________________________
void MainWindow::open( const FileRecord& constRecord )
{

    Debug::Throw( "MainWindow::open.\n" );

    FileRecord record( constRecord );
    if( record.file().isEmpty() )
    {
        QString file( FileDialog( this ).selectFile( fileSelectionFrame_->path() ).getFile() );
        if( file.isNull() ) return;
        else record = FileRecord( file );
    }

    // update file selection frame
    fileSelectionFrame_->setFile( record.file() );

    // update fileEditor and menu
    _setFile( record.file() );

}

//_______________________________________________________
void MainWindow::clearCache( void )
{
    cache_.clear();
    fileSelectionFrame_->clearCache();
    revisionFrame_->clearCache();
}

//____________________________________
void MainWindow::closeEvent( QCloseEvent *event )
{
    Debug::Throw( "MainWindow::closeEvent.\n" );
    _saveConfiguration();
    event->accept();
    qApp->quit();
}

//_______________________________________
void MainWindow::_updateConfiguration( void )
{

    Debug::Throw( "MainWindow::_updateConfiguration" );

    // command dialog
    if( XmlOptions::get().get<bool>( "SHOW_COMMAND_LIST" ) ) _showCommandDialog();
    else commandDialog().hide();

    // log dialog
    if( XmlOptions::get().get<bool>( "SHOW_LOG" ) ) _showLogDialog();
    else logDialog().hide();

}

///_______________________________________
void MainWindow::_saveConfiguration( void )
{

    Debug::Throw( "MainWindow::_saveConfiguration.\n" );

    // retrieve cache from children and write to file
    const qreal fileInfoScale( 1024.0*1024.0/182 );
    const qreal revisionScale( 1024.0*1024.0/153 );

    const int maxFileInfoCacheSize( fileInfoScale*XmlOptions::get().get<int>("FILEINFO_CACHE_SIZE") );
    const int maxRevisionCacheSize( revisionScale*XmlOptions::get().get<int>("REVISION_CACHE_SIZE") );
    cache_.setFileInfoCache( fileSelectionFrame_->cache().setMaximumSize( XmlOptions::get().get<bool>( "LIMIT_FILEINFO_CACHE" ) ? maxFileInfoCacheSize : -1 ).truncate() );
    cache_.setRevisionCache( revisionFrame_->cache().setMaximumSize( XmlOptions::get().get<bool>( "LIMIT_REVISION_CACHE" ) ? maxRevisionCacheSize:-1 ).truncate() );
    cache_.write();

}

//__________________________________________________________________________
void MainWindow::_reload( void )
{

    Debug::Throw( "MainWindow::_reload.\n" );

    // reload information based on editor contents
    QFileInfo file( fileSelectionFrame_->editor().path() );
    QFileInfo path( file.absolutePath() );
    if( path.exists() )
    {

        Debug::Throw() << "MainWindow::_reload - file: " << path.absoluteFilePath() << endl;
        fileSelectionFrame_->setFile( file.absoluteFilePath() );

    } else {

        // invalid path. Prompt dialog
        const QString buffer = QString( tr( "Path '%1' is invalid. <Reload> canceled." ) ).arg( path.absoluteFilePath() );
        InformationDialog( this, buffer ).exec();

    }

}

//__________________________________________________________________________
void MainWindow::_setFile( const File& file )
{

    Debug::Throw() << "MainWindow::_setFile - file: " << file << endl;

    // update window title
    {

        File path( file );
        if( !path.isDirectory() ) path = path.path();
        if( path.endsWith('/') ) path = path.left( path.size()-1 );

        QString buffer;
        if( !path.isEmpty() ) buffer = QString( "%1 - Cvswatch" ).arg( path.localName() );
        else buffer = "Cvswatch";

        setWindowTitle( buffer );

    }

    // clear comments if file differ from current revision file
    if( file != revisionFrame_->file() )
    { revisionFrame_->setFile( file.isDirectory() ? File():file ); }

    // add to openPrevious menu
    Singleton::get().application<Application>()->recentFiles().add( file.isDirectory() ? file:file.path() );
    menu().recentFilesMenu().setCurrentFile( file.isDirectory() ? file:file.path() );

    return;
}

//____________________________________________________________________________________________
void MainWindow::_placeSelected( const BaseFileInfo& fileInfo )
{
    Debug::Throw() << "MainWindow::_placeSelected - file: " << fileInfo << endl;
    if( !fileInfo.isFolder() || fileInfo.file().isEmpty() ) return;
    fileSelectionFrame_->setFile( fileInfo.file() );
    _setFile( fileInfo.file() );
    return;
}

//_______________________________________________________________________
void MainWindow::_updateStatusBar( const CvsCommand& command )
{
    Debug::Throw( "MainWindow::_updateStatusBar.\n" );

    // create stream, add name, and possible argument
    QString buffer;
    QTextStream what( &buffer );
    what << command.name();

    if( command.argumentCount() )
    { what << " " << command.argument(0); }

    // update status bar
    statusbar_->label(0).setText( buffer );

}
//__________________________________________________________________________
void MainWindow::_showCommandDialog( void )
{

    Debug::Throw( "MainWindow::_showCommandDialog.\n" );
    XmlOptions::get().set<bool>( "SHOW_COMMAND_LIST", true );
    commandDialog().show();
    commandDialog().raise();

}

//_______________________________________________
void MainWindow::_commandDialogHidden( void )
{
    Debug::Throw( "Application::_commandDialogHidden.\n" );
    XmlOptions::get().set<bool>( "SHOW_COMMAND_LIST", false );
}

//__________________________________________________________________________
void MainWindow::_showLogDialog( void )
{

    Debug::Throw( "MainWindow::_showLogDialog.\n" );
    XmlOptions::get().set<bool>( "SHOW_LOG", true );
    logDialog().show();
    logDialog().raise();

}

//_______________________________________________
void MainWindow::_logDialogHidden( void )
{
    Debug::Throw( "MainWindow::_logDialogHidden.\n" );
    XmlOptions::get().set<bool>( "SHOW_LOG", false );
}

//_______________________________________________________________________
void MainWindow::_showFileInfoCache( void )
{

    Debug::Throw( "MainWindow::_showFileInfoCache.\n" );
    FileInfoCacheDialog dialog( this );
    dialog.setWindowTitle( tr( "File Information - Cvswatch" ) );
    dialog.setCache( cache_.fileInfoCache() );
    dialog.centerOnParent().exec();

}

//_______________________________________________________________________
void MainWindow::_showRevisionCache( void )
{

    Debug::Throw( "MainWindow::_showRevisionCache.\n" );
    RevisionCacheDialog dialog( this );
    dialog.setWindowTitle( tr( "Revisions - Cvswatch" ) );
    dialog.setCache( cache_.revisionCache() );
    dialog.centerOnParent().exec();

}

//_______________________________________________
void MainWindow::_fileInformation( const FileInfo& fileInfo, const QIcon& icon )
{

    FileInformationDialog dialog( this, fileInfo );
    dialog.setIcon( icon );
    dialog.exec();

}

//_______________________________________________________________________
void MainWindow::_installActions( void )
{
    Debug::Throw( "MainWindow::_installActions.\n" );

    addAction( reloadAction_ = new QAction( IconEngine::get( IconNames::Reload ), tr( "Reload" ), this ) );
    reloadAction_->setShortcut( QKeySequence::Refresh );
    connect( reloadAction_, SIGNAL(triggered()), SLOT(_reload()) );

    addAction( openAction_ = new QAction( IconEngine::get( IconNames::Open ), tr( "Open..." ), this ) );
    openAction_->setShortcut( QKeySequence::Open );
    openAction_->setToolTip( tr( "Select file and directory" ) );
    connect( openAction_, SIGNAL(triggered()), SLOT(open()) );

    addAction( fileInfoCacheAction_ = new QAction( tr( "File Cache" ), this ) );
    fileInfoCacheAction_->setToolTip( tr( "Show file cache" ) );
    connect( fileInfoCacheAction_, SIGNAL(triggered()), SLOT(_showFileInfoCache()) );

    addAction( revisionCacheAction_ = new QAction( tr( "Revision Cache" ), this ) );
    revisionCacheAction_->setToolTip( tr( "Show revision cache" ) );
    connect( revisionCacheAction_, SIGNAL(triggered()), SLOT(_showRevisionCache()) );

    addAction( showCommandDialogAction_ = new QAction( IconEngine::get( IconNames::Find ), tr( "Show Command List" ), this ) );
    connect( showCommandDialogAction_, SIGNAL(triggered()), SLOT(_showCommandDialog()) );

    addAction( showLogDialogAction_ = new QAction( IconEngine::get( IconNames::Find ), tr( "Show Log" ), this ) );
    connect( showLogDialogAction_, SIGNAL(triggered()), SLOT(_showLogDialog()) );

}
