#ifndef MainWindow_h
#define MainWindow_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "BaseMainWindow.h"
#include "BaseFileInfo.h"
#include "BusyWidget.h"
#include "Cache.h"
#include "Counter.h"
#include "CvsCommand.h"
#include "Debug.h"
#include "FileRecord.h"
#include "Revision.h"
#include "TextEditor.h"

#include <QBasicTimer>
#include <QTimerEvent>
#include <QAction>

class BaseStatusBar;
class CvsCommandDialog;
class DockWidget;
class FileIconProvider;
class FileSelectionFrame;
class LogDialog;
class Menu;
class PlacesWidget;
class RevisionFrame;

//! cvs log revision frame view
class MainWindow: public BaseMainWindow, public Counter
{

    //! Qt meta object declaration
    Q_OBJECT

    public:

    //! constructor
    MainWindow( QWidget *parent = 0 );

    //! menu
    Menu& menu( void ) const
    { return *menu_; }

    //! info list
    FileSelectionFrame& selectionFrame( void ) const
    { return *fileSelectionFrame_; }

    //! info list
    RevisionFrame& revisionFrame( void ) const
    { return *revisionFrame_; }

    //! command list
    CvsCommandDialog& commandDialog( void ) const
    { return *commandDialog_; }

    //! log dialog
    LogDialog& logDialog( void ) const
    { return *logDialog_; }

    //!@name actions
    //@{

    //! reload action
    QAction& reloadAction( void ) const
    { return *reloadAction_; }

    //! open
    QAction& openAction( void ) const
    { return *openAction_; }

    //! toggle command frame
    QAction& showCommandDialogAction( void ) const
    { return *showCommandDialogAction_; }

    //! toggle log visibility
    QAction& showLogDialogAction( void ) const
    { return *showLogDialogAction_; }

    //! file info cache
    QAction& fileInfoCacheAction( void ) const
    { return *fileInfoCacheAction_; }

    //! revision cache
    QAction& revisionCacheAction( void ) const
    { return *revisionCacheAction_; }

    //@}

    public Q_SLOTS:

    //! display cvs revisions for a given file
    void open( const FileRecord& = FileRecord() );

    //! clear cache
    void clearCache( void );

    //! busy
    virtual void busy( void )
    { busyWidget_->start(); }

    //! busy
    virtual void idle( void )
    { busyWidget_->stop(); }

    protected:

    //! close window event handler
    void closeEvent( QCloseEvent* );

    protected Q_SLOTS:

    //! save configuration
    void _saveConfiguration( void );

    //! reload selection frame using file editor
    void _reload( void );

    //! select file from FileSelectionFrame
    void _setFile( const File& );

    //! file selected in one of the selection frame
    void _placeSelected( const BaseFileInfo& );

    //! update status bar from current command
    void _updateStatusBar( const CvsCommand& );

    //! toggle command frame visibility
    void _showCommandDialog( void );

    //! command dialog hidden
    void _commandDialogHidden( void );

    //! toggle log visibility
    void _showLogDialog( void );

    //! log dialog hidden
    void _logDialogHidden( void );

    //! show file info cache
    void _showFileInfoCache( void );

    //! show revision cache
    void _showRevisionCache( void );

    //! show file information
    void _fileInformation( const FileInfo&, const QIcon& );

    private Q_SLOTS:

    //! update configuration
    void _updateConfiguration( void );

    private:

    //! install actions
    void _installActions( void );

    //! icon provider
    FileIconProvider* iconProvider_;

    //! revisions and fileList cache
    Cache cache_;

    //! menu
    Menu* menu_;

    //! directory tree navigation
    FileSelectionFrame* fileSelectionFrame_;

    //! list of log revisions
    RevisionFrame* revisionFrame_;

    //! command list
    CvsCommandDialog* commandDialog_;

    //! log
    LogDialog* logDialog_;

    //! state frame
    BaseStatusBar* statusbar_;

    //! command list size
    QSize commandDialogSize_;

    //! busy widget
    BusyWidget* busyWidget_;

    //! places
    PlacesWidget* placesWidget_;

    //! dock widget
    DockWidget* placesPanel_;

    //!@name actions
    //@{

    //! reload
    QAction* reloadAction_;

    //! open
    QAction* openAction_;

    //! toggle command frame action
    QAction* showCommandDialogAction_;

    //! toggle log action
    QAction* showLogDialogAction_;

    //! file info cache action
    QAction* fileInfoCacheAction_;

    //! revision cache action
    QAction* revisionCacheAction_;

    //@}

};

#endif
