/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "RevisionDelegate.h"

#include "Debug.h"
#include "RevisionModel.h"
#include "Revision.h"

#include <QAbstractScrollArea>
#include <QAbstractTextDocumentLayout>
#include <QApplication>
#include <QStyleOptionViewItem>
#include <QPainter>

//_________________________________________________________
RevisionDelegate::RevisionDelegate( QObject *parent ):
    QAbstractItemDelegate( parent ),
    Counter( "RevisionDelegate" )
{ Debug::Throw( "RevisionDelegate::RevisionDelegate.\n" ); }

//_________________________________________________________
void RevisionDelegate::paint( QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index ) const
{

    // check index
    if ( !index.isValid() ) return;

    // cast model
    const RevisionModel& model( *static_cast<const RevisionModel*>( index.model() ) );
    if( !model.contains( index ) ) return;

    // selection
    {
        QStyleOptionViewItemV4 viewOption(option);
        viewOption.showDecorationSelected = true;
        QStyle *style = viewOption.widget ? viewOption.widget->style() : QApplication::style();
        style->drawPrimitive(QStyle::PE_PanelItemViewItem, &viewOption, painter, viewOption.widget);
    }

    // get revision
    const Revision& revision( model.get( index ) );

    // create layout
    QTextDocument document;
    document.setDefaultFont( option.font );
    document.setTextWidth( option.rect.width() );

    QTextOption textOption;
    textOption.setWrapMode( QTextOption::WordWrap );
    document.setDefaultTextOption( textOption );

    QTextCursor cursor( &document );

    // modify color
    QPalette::ColorGroup colorGroup = option.state & QStyle::State_Enabled ? QPalette::Normal : QPalette::Disabled;
    if( colorGroup == QPalette::Normal && !(option.state & QStyle::State_Active) )
    { colorGroup = QPalette::Inactive; }

    QColor foreground;
    if( option.state & QStyle::State_Selected ) foreground = option.palette.color( colorGroup, QPalette::HighlightedText );
    else foreground = option.palette.color( colorGroup, QPalette::Text );

    QTextCharFormat charFormat( cursor.charFormat() );
    charFormat.setForeground( foreground );
    cursor.setCharFormat( charFormat );
    _layoutText( cursor, revision );

    painter->save();
    painter->translate( option.rect.topLeft() );
    painter->setClipRect( QRect( QPoint(0,0), option.rect.size() ) );
    document.drawContents( painter );
    painter->restore();

    // focus
    _drawFocus( painter, option, option.rect );

}

//_________________________________________________________
QSize RevisionDelegate::sizeHint( const QStyleOptionViewItem &option, const QModelIndex &index ) const
{

    if ( !index.isValid() ) return QSize();

    // cast model
    const RevisionModel& model( *static_cast<const RevisionModel*>( index.model() ) );
    if( !model.contains( index ) ) return QSize();

    // get revision
    Revision revision( model.get( index ) );

    // create document and layout text
    QTextDocument document;
    document.setDefaultFont( option.font );
    document.setTextWidth( static_cast<QAbstractScrollArea*>( parent() )->viewport()->width() );

    QTextOption textOption;
    textOption.setWrapMode( QTextOption::WordWrap );
    document.setDefaultTextOption( textOption );

    QTextCursor cursor( &document );
    _layoutText( cursor, revision );

    // size
    return QSize( document.idealWidth(), document.size().height() );

}

//_________________________________________________________
void RevisionDelegate::_layoutText( QTextCursor& cursor, const Revision& revision ) const
{

    QTextCharFormat defaultFormat( cursor.charFormat() );
    QTextCharFormat boldFormat( defaultFormat );
    boldFormat.setFontWeight( QFont::Bold );

    // revision number
    cursor.insertText( tr( "Revision: " ), defaultFormat );
    cursor.insertText( revision.name() + "\n", boldFormat );

    // date
    cursor.insertText( tr( "Date: " ), defaultFormat );
    cursor.insertText( revision.timeStamp().toString() + "\n", boldFormat );

    // author
    cursor.insertText( tr( "Author: " ), defaultFormat );
    cursor.insertText( revision.author() + "\n", boldFormat );

    // message
    cursor.insertText( revision.comments().trimmed(), defaultFormat );

}

//_________________________________________________________
void RevisionDelegate::_drawFocus( QPainter *painter, const QStyleOptionViewItem &option, const QRect &rect ) const
{
    if (option.state & QStyle::State_HasFocus)
    {

        QStyleOptionFocusRect o;
        o.QStyleOption::operator=(option);
        o.rect = rect;
        o.state |= QStyle::State_KeyboardFocusChange;
        QPalette::ColorGroup colorGroup = (option.state & QStyle::State_Enabled) ? QPalette::Normal : QPalette::Disabled;
        o.backgroundColor = option.palette.color( colorGroup, (option.state & QStyle::State_Selected) ? QPalette::Highlight : QPalette::Background );

        QApplication::style()->drawPrimitive( QStyle::PE_FrameFocusRect, &o, painter );

    }

}
