#ifndef RevisionFrame_h
#define RevisionFrame_h

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "CvsCommand.h"
#include "Debug.h"
#include "File.h"
#include "Revision.h"
#include "RevisionCache.h"
#include "RevisionModel.h"
#include "XmlOptions.h"

#include <QBasicTimer>
#include <QResizeEvent>
#include <QWidget>

class TreeView;
class RevisionDelegate;
class CvsInterface;

//* cvs log version list
class RevisionFrame: public QWidget, public Counter
{

    Q_OBJECT

    public:

    //* constructor
    RevisionFrame( QWidget* );

    //* default size
    void setDefaultWidth( const int& value )
    { defaultWidth_ = value; }

    //* default width
    const int& defaultWidth( void ) const
    { return defaultWidth_; }

    //* size
    QSize sizeHint( void ) const
    { return defaultWidth_ >= 0 ? QSize( defaultWidth_, 0 ):QWidget::sizeHint(); }

    //* clear
    void clear( void );

    //* get file
    const File& file( void )
    { return file_; }

    //* cvs interface
    CvsInterface& interface( void ) const
    { return *interface_; }

    //* revision list
    TreeView& list( void )
    { return *list_; }

    //* cache
    RevisionCache& cache( void )
    {

        if( !localCache_.empty() )
        {
            std::sort( localCache_.begin(), localCache_.end() );
            localCache_.erase( std::unique( localCache_.begin(), localCache_.end() ), localCache_.end() );
            cache_[file()] = localCache_;
        }
        return cache_;
    }

    //* cache
    void setCache( const RevisionCache& cache )
    { cache_ = cache; }

    //*@name  actions
    //@{

    //* toggle visibility action
    QAction& visibilityAction( void ) const
    { return *visibilityAction_; }

    //* edit local
    QAction& editLocalAction( void ) const
    { return *editLocalAction_; }

    //* edit revision
    QAction& editRevisionAction( void ) const
    { return *editRevisionAction_; }

    //* diff local and revision
    QAction& diffAction( void ) const
    { return *diffAction_; }

    //@}

    Q_SIGNALS:

    //* selection changed
    void selectionChanged( void );

    //* current revision changed
    void currentRevisionChanged( const Revision& );

    //* item resized
    void itemResized( const QModelIndex& );

    public Q_SLOTS:

    //* set file
    void setFile( const File& file );

    //* cvs command completed
    void commandCompleted( const CvsCommand& );

    //* clear caches
    void clearCache( void )
    {
        cache_.clear();
        localCache_.clear();
    }

    protected:

    //* timer event
    virtual void timerEvent( QTimerEvent* );

    //* resize event
    virtual void resizeEvent( QResizeEvent* );

    //* reload revisions using CVS
    void _reload( void );

    protected Q_SLOTS:

    //* update actions based on selection and contents
    void _updateActions( void );

    //* edit local file
    void _editLocal( void );

    //* edit revision
    void _editRevision( void );

    //* diff revision to local
    void _diff( void );

    //* current revision changed
    void _currentRevisionChanged( const QModelIndex& index )
    { if( index.isValid() ) emit currentRevisionChanged( model_.get( index ) ); }

    //* toggle visibility
    void _toggleVisibility( bool value )
    {
        setVisible( value );
        XmlOptions::get().set<bool>( visibilityOptionName_, value );
    }

    //*@name slots triggered by CvsInterface
    //@{

    //* display revisions in list
    void _showRevisions( File, Revision::List );

    //* edit revision
    void _editRevision( File source, File destination );

    //* diff revision to local
    void _diff( File source, File destination );

    //@}

    private:

    //* install actions
    void _installActions( void );

    //* display revisions in list
    void _showRevisionsFromLocalCache( void );

    //* default width;
    int defaultWidth_ = 0;

    //* revision list
    TreeView* list_ = nullptr;

    //* revision delegate
    RevisionDelegate* delegate_ = nullptr;

    //* model
    RevisionModel model_;

    //* associated file
    File file_;

    //* timer
    /**
    used to retrieve log from CVS interface,
    with some delay to avoid running too many
    CVS commands when parsing the tree */
    QBasicTimer timer_;

    //* resize timer
    QBasicTimer resizeTimer_;

    //* revision cache
    RevisionCache cache_;

    //* local revision cache
    Revision::List localCache_;

    //* cvs interface
    CvsInterface* interface_ = nullptr;

    //*@name actions
    //@{

    //* visibility option name
    QAction* visibilityAction_ = nullptr;

    //* edit local file action
    QAction* editLocalAction_ = nullptr;

    //* edit revision
    QAction* editRevisionAction_ = nullptr;

    //* diff
    QAction* diffAction_ = nullptr;

    //@}

    //* visibility option name
    QString visibilityOptionName_;

};

#endif
